import time
import pickle
import base64
import os
try:
    from configparser import NoOptionError
except ImportError:
    from  ConfigParser import NoOptionError

from temboardagent.routing import add_route, add_worker
from temboardagent.api_wrapper import *
from temboardagent.logger import set_logger_name, get_logger, get_tb
from temboardagent.configuration import (PluginConfiguration, ConfigurationError,
                                    Configuration)
from temboardagent.tools import validate_parameters
from temboardagent.types import *
from temboardagent.sharedmemory import Command
from temboardagent.tools import hash_id
from temboardagent.errors import (HTTPError, SharedItem_exists,
                SharedItem_no_free_slot_left, SharedItem_not_found, NotificationError)
from temboardagent.workers import COMMAND_START, COMMAND_DONE, COMMAND_ERROR
from temboardagent.spc import connector, error
from temboardagent.command import exec_command, oneline_cmd_to_array, exec_script
from temboardagent.notification import NotificationMgmt, Notification

import administration.functions as admin_functions
from administration.types import *

@add_route('GET', '/administration/pg_version')
def api_pg_version(http_context, queue_in = None, config = None, sessions = None, commands = None):
    set_logger_name("administration")
    return api_function_wrapper_pg(config, http_context, sessions, admin_functions, 'pg_version')

@add_route('POST', '/administration/control')
def post_pg_control(http_context, queue_in = None, config = None, sessions = None, commands = None):
    # NOTE: in this case we don't want to use api functions wrapper, it leads
    # to "Broken pipe" error with debian init.d on start/restart. This is
    # probably due to getattr() call.
    set_logger_name("administration")
    # Get a new logger.
    logger = get_logger(config)
    post = http_context['post']

    try:
        check_sessionid(http_context['headers'], sessions)
        # Check POST parameters.
        validate_parameters(post, [
            ('action', T_CONTROL, False)
        ])
        session = sessions.get_by_sessionid(http_context['headers']['X-Session'].encode('utf-8'))
    except (Exception, HTTPError) as e:
        logger.traceback(get_bt())
        logger.error(str(e))
        logger.debug(http_context)
        if isinstance(e, HTTPError):
            raise e
        else:
            raise HTTPError(500, "Internal error.")

    try:
        NotificationMgmt.push(config, Notification(
                                        username = session.username,
                                        message = "PostgreSQL %s" % (post['action'])))
    except (NotificationError, Exception) as e:
        logger.traceback(get_tb())
        logger.error(str(e))

    try:
        logger.info("PostgreSQL '%s' requested." % (post['action']))
        cmd_args = oneline_cmd_to_array(config.plugins['administration']['pg_ctl'] % (post['action']))
        (rcode, stdout, stderr) = exec_script(cmd_args)
        if rcode != 0:
            raise Exception(str(stderr))
        # Let's check if postgresql is up & running on 'start' or 'restart' action.
        if post['action'] in ['start', 'restart']:
            conn = connector(
                host = config.postgresql['host'],
                port = config.postgresql['port'],
                user = config.postgresql['user'],
                password = config.postgresql['password'],
                database = config.postgresql['dbname']
            )
            # When a start/restart operation is requested, after the startup/pg_ctl
            # script is executed we check that postgres is up & running: while the
            # PG connection is not working, during 10 seconds (max) we'll check
            # (connect/SELECT 1/disconnect) the connection, every 0.5 second.
            retry = True
            t_start = time.time()
            while retry:
                try:
                    conn.connect()
                    conn.execute('SELECT 1')
                    conn.close()
                    logger.info("Done.")
                    return {'action': post['action'], 'state': 'ok'}
                except error as e:
                    if (time.time() - t_start) > 10:
                        try:
                            conn.close()
                        except error as e:
                            pass
                        except Exception:
                            pass
                        logger.info("Failed.")
                        return {'action': post['action'], 'state': 'ko'}
                time.sleep(0.5)

        elif post['action'] == 'stop':
            conn = connector(
                host = config.postgresql['host'],
                port = config.postgresql['port'],
                user = config.postgresql['user'],
                password = config.postgresql['password'],
                database = config.postgresql['dbname']
            )
            # Check the PG conn is not working anymore.
            try:
                retry = True
                t_start = time.time()
                while retry:
                    conn.connect()
                    conn.execute('SELECT 1')
                    conn.close()
                    time.sleep(0.5)
                    if (time.time() - t_start) > 10:
                        retry = False
                logger.info("Failed.")
                return {'action': post['action'], 'state': 'ko'}
            except error as e:
                logger.info("Done.")
                return {'action': post['action'], 'state': 'ok'}
        logger.info("Done.")
        return {'action': post['action'], 'state': 'ok'}
    except (Exception, error, HTTPError) as e:
        logger.traceback(get_tb())
        logger.error(str(e))
        logger.info("Failed")
        if isinstance(e, HTTPError):
            raise e
        else:
            raise HTTPError(500, "Internal error.")

@add_route('POST', '/administration/vacuum')
def api_vacuum(http_context, queue_in = None, config = None, sessions = None, commands = None):
    set_logger_name("administration")
    worker = b'vacuum'
    # Get a new logger.
    logger = get_logger(config)
    try:
        check_sessionid(http_context['headers'], sessions)
        post = http_context['post']
        # Check POST parameters.
        validate_parameters(post, [
            ('database', T_OBJECTNAME, False),
            ('table', T_OBJECTNAME, False),
            ('mode', T_VACUUMMODE, False)
        ])
        # Serialize parameters.
        parameters = base64.b64encode(
                        pickle.dumps({
                            'database': post['database'],
                            'table': post['table'],
                            'mode': post['mode']
        })).decode('utf-8')
    except (Exception, HTTPError) as e:
        logger.traceback(get_tb())
        logger.error(str(e))
        if isinstance(e, HTTPError):
            raise e
        else:
            raise HTTPError(500, "Internal error.")

    # Check command uniqueness.
    try:
        commands.check_uniqueness(worker, parameters)
    except SharedItem_exists as e:
        logger.traceback(get_tb())
        logger.error(str(e))
        raise HTTPError(402, "Vaccum '%s' already running on table '%s'." % (post['mode'], post['table']))
    cid =  hash_id(worker + b'-' + parameters.encode('utf-8'))
    command =  Command(
            cid.encode('utf-8'),
            time.time(),
            0,
            worker,
            parameters,
            0,
            u'')
    try:
        commands.add(command)
        # Put the Command in the command queue
        queue_in.put(command)
        return {"cid": cid}
    except SharedItem_no_free_slot_left as e:
        logger.traceback(get_tb())
        logger.error(str(e))
        raise HTTPError(500, "Internal error.")

@add_worker(b'vacuum')
def worker_vacuum(commands, command, config):
    start_time = time.time() * 1000
    set_logger_name("vacuum_worker")
    logger = get_logger(config)
    logger.info("Starting with pid=%s" % (os.getpid()))
    logger.debug("commandid=%s" % (command.commandid,))

    try:
        command.state = COMMAND_START
        command.time = time.time()
        commands.update(command)
        parameters = pickle.loads(base64.b64decode(command.parameters))
        logger.debug("table=%s, mode=%s, database=%s"
                    % (parameters['table'], parameters['mode'], parameters['database'],))

        conn = connector(
            host = config.postgresql['host'],
            port = config.postgresql['port'],
            user = config.postgresql['user'],
            password = config.postgresql['password'],
            database = parameters['database']
        )
        conn.connect()
        if parameters['mode'] == 'standard':
            query = "VACUUM %s" % (parameters['table'],)
        else:
            query = "VACUUM %s %s" % (parameters['mode'], parameters['table'],)
        conn.execute(query)
        conn.close()
    except (error, SharedItem_not_found, Exception)  as e:
        command.state = COMMAND_ERROR
        command.result = str(e)
        command.time = time.time()

        logger.traceback(get_tb())
        logger.error(str(e))

        try:
            commands.update(command)
            conn.close()
        except Exception as e:
            pass
        logger.info("Failed.")
        return

    try:
        command.state = COMMAND_DONE
        command.time = time.time()
        commands.update(command)
    except Exception as e:
        logger.traceback(get_tb())
        logger.error(str(e))

    logger.info("Done.")
    logger.debug(" in %s s." % (str((time.time()*1000 - start_time)/1000),))

def configuration(config):
    class Configuration(PluginConfiguration):
        def __init__(self, config, *args, **kwargs):
            PluginConfiguration.__init__(self, config.configfile, *args, **kwargs)

            self.plugin_configuration = {
                'pg_ctl': None,
            }
            set_logger_name("administration")
            logger = get_logger(config)

            try:
                self.check_section(__name__)
            except ConfigurationError as e:
                return

            try:
                val =  self.get(__name__, 'pg_ctl')
                for char in ['"', '\'']:
                    if val.startswith(char) and val.endswith(char):
                        val = val[1:-1]
                self.plugin_configuration['pg_ctl'] = val
            except NoOptionError:
                pass

    c = Configuration(config)
    return c.plugin_configuration



import time
import signal
import json
from os import getpid
try:
    from configparser import NoOptionError
except ImportError:
    from  ConfigParser import NoOptionError

from temboardagent.routing import add_route
from temboardagent.api_wrapper import *
from temboardagent.logger import set_logger_name, get_tb
from temboardagent.spc import connector, error
from temboardagent.routing import add_route, add_worker
from temboardagent.configuration import (PluginConfiguration, ConfigurationError,
                                    Configuration)
from temboardagent.logger import get_logger, set_logger_name, get_tb
from temboardagent.sharedmemory import Command
from temboardagent.tools import hash_id
from temboardagent.errors import (HTTPError, SharedItem_exists,
                SharedItem_no_free_slot_left, SharedItem_not_found)
from temboardagent.workers import COMMAND_START, COMMAND_DONE, COMMAND_ERROR
from temboardagent.pluginsmgmt import load_plugins_configurations
from temboardagent.queue import Queue, Message
import dashboard.metrics

@add_route('GET', '/dashboard')
def dashboard(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard Fetch all
    @apiVersion 0.0.1
    @apiName GetDasboard
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object} active_backends
    @apiSuccess {Number} active_backends.nb Number of PostgreSQL active backends.
    @apiSuccess {Number} active_backends.time Timestamp (floating with ms) when the number of backend has been calculated.
    @apiSuccess {Number} loadaverage System loadaverage.
    @apiSuccess {String} os_version Operating system version.
    @apiSuccess {String} os_version PostgreSQL version.
    @apiSuccess {Number} hitratio PostgreSQL cache/hit ratio (%)
    @apiSuccess {Number} n_cpu Number of CPU.
    @apiSuccess {Object} databases
    @apiSuccess {String} databases.total_size PostgreSQL instance total size (with unit).
    @apiSuccess {String} databases.time Time when DBs informations have been retreived (HH:MM).
    @apiSuccess {Number} databases.databases Number of databases.
    @apiSuccess {Number} databases.total_commit Number of commited xact.
    @apiSuccess {Number} databases.total_rollback Number of rollbacked xact.
    @apiSuccess {Number} databases.timestamp Timestamp (floating with ms).
    @apiSuccess {Object} memory
    @apiSuccess {Number} memory.total Total amount of memory (kB).
    @apiSuccess {Number} memory.active Active memory (%).
    @apiSuccess {Number} memory.cached Memory used as OS cache (%).
    @apiSuccess {Number} memory.free Unused memory (%).
    @apiSuccess {String} hostname Machine hostname.
    @apiSuccess {Object} cpu
    @apiSuccess {Number} cpu.iowait CPU cycles waiting for I/O operation (%).
    @apiSuccess {Number} cpu.idle CPU IDLE time (%).
    @apiSuccess {Number} cpu.steal CPU steal time (%).
    @apiSuccess {Number} cpu.user CPU user time (%).
    @apiSuccess {Number} cpu.system CPU system time (%).
    @apiSuccess {Object} buffers
    @apiSuccess {Number} buffers.nb Allocated buffers from PostgreSQL bgwriter.
    @apiSuccess {Number} buffers.time Timestamp (floating with ms) when the buffers number has been fetched.

    @apiExample {curl} Example usage:
        curl -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" http://localhost:12345/dashboard

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:57:52 GMT
        Content-type: application/json
        {
            "active_backends":
            {
                "nb": 1,
                "time": 1429617751.29224
            },
            "loadaverage": 0.28,
            "os_version": "Linux 3.16.0-34-generic x86_64",
            "pg_version": "9.4.1",
            "n_cpu": "4",
            "hitratio": 98.0,
            "databases":
            {
                "total_size": "1242 MB",
                "time": "14:02",
                "databases": 4,
                "total_commit": 16728291,
                "total_rollback": 873
            },
            "memory": {
                "total": 3950660,
                "active": 46.9,
                "cached": 20.2,
                "free": 32.9
            },
            "hostname": "neptune",
            "cpu":
            {
                "iowait": 0.0,
                "idle": 97.5,
                "steal": 0.0,
                "user": 2.5,
                "system": 0.0
            },
            "buffers":
            {
                "nb": 348247,
                "time": 1429617751.276508
            }
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper(config, http_context, sessions, metrics, 'get_metrics_queue')

@add_route('GET', '/dashboard/live')
def dashboard_live(http_context, queue_in = None, config = None, sessions = None, commands = None):
    set_logger_name("dashboard")
    return api_function_wrapper_pg(config, http_context, sessions, metrics, 'get_metrics')

@add_route('GET', '/dashboard/history')
def dashboard_history(http_context, queue_in = None, config = None, sessions = None, commands = None):
    set_logger_name("dashboard")
    return api_function_wrapper(config, http_context, sessions, metrics, 'get_history_metrics_queue')

@add_route('GET', '/dashboard/buffers')
def dashboard_buffers(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/buffers PostgreSQL bgwriter buffers
    @apiVersion 0.0.1
    @apiName GetDasboardBuffers
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object} buffers
    @apiSuccess {Number} buffers.nb Allocated buffers from PostgreSQL bgwriter.
    @apiSuccess {Number} buffers.time Timestamp (floating with ms) when the buffers number has been fetched.

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/buffers

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:22:50 GMT
        Content-type: application/json

        {"buffers": {"nb": 348247, "time": 1429701770.303621}}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper_pg(config, http_context, sessions, metrics, 'get_buffers')

@add_route('GET', '/dashboard/hitratio')
def dashboard_hitratio(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/hitratio PostgreSQL cache hit ratio
    @apiVersion 0.0.1
    @apiName GetDasboardHitratio
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Number} hitratio PostgreSQL global cache/hit ratio (%)

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/hitratio

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:27:27 GMT
        Content-type: application/json

        {"hitratio": 98.0}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper_pg(config, http_context, sessions, metrics, 'get_hitratio')

@add_route('GET', '/dashboard/active_backends')
def dashboard_active_backends(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/active_backends PostgreSQL active backends
    @apiVersion 0.0.1
    @apiName GetDasboardActiveBackends
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object} active_backends
    @apiSuccess {Number} active_backends.nb Number of PostgreSQL active backends.
    @apiSuccess {Number} active_backends.time Timestamp (floating with ms) when the number of backend has been calculated.

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/active_backends

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:38:02 GMT
        Content-type: application/json

        {"active_backends": {"nb": 1, "time": 1429702682.228157}}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper_pg(config, http_context, sessions, metrics, 'get_active_backends')

@add_route('GET', '/dashboard/cpu')
def dashboard_cpu(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/cpu CPU usage
    @apiVersion 0.0.1
    @apiName GetDasboardCPU
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object} cpu
    @apiSuccess {Number} cpu.iowait CPU cycles waiting for I/O operation (%).
    @apiSuccess {Number} cpu.idle CPU IDLE time (%).
    @apiSuccess {Number} cpu.steal CPU steal time (%).
    @apiSuccess {Number} cpu.user CPU user time (%).
    @apiSuccess {Number} cpu.system CPU system time (%).

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/cpu

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:39:58 GMT
        Content-type: application/json

        {"cpu": {"iowait": 0.0, "idle": 97.5, "steal": 0.0, "user": 2.5, "system": 0.0}}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper(config, http_context, sessions, metrics, 'get_cpu_usage')

@add_route('GET', '/dashboard/loadaverage')
def dashboard_loadaverage(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/loadaverage System loadaverage
    @apiVersion 0.0.1
    @apiName GetDasboardLoadaverage
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Number} loadaverage System loadaverage.

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/loadaverage

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:41:31 GMT
        Content-type: application/json

        {"loadaverage": 0.31}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    return api_function_wrapper(config, http_context, sessions, metrics, 'get_loadaverage')

@add_route('GET', '/dashboard/memory')
def dashboard_memory(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/memory Memory usage
    @apiVersion 0.0.1
    @apiName GetDasboardMemory
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object} memory
    @apiSuccess {Number} memory.total Total amount of memory (kB).
    @apiSuccess {Number} memory.active Active memory (%).
    @apiSuccess {Number} memory.cached Memory used as OS cache (%).
    @apiSuccess {Number} memory.free Unused memory (%).

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/memory

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:44:14 GMT
        Content-type: application/json

        {"memory": {"total": 3950660,"active": 56.6, "cached": 33.8, "free": 9.6}}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper(config, http_context, sessions, metrics, 'get_memory_usage')

@add_route('GET', '/dashboard/hostname')
def dashboard_hostname(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/hostname Machine hostname
    @apiVersion 0.0.1
    @apiName GetDasboardHostname
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {String} hostname Machine hostname.

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/hostname

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:47:34 GMT
        Content-type: application/json

        {"hostname": "neptune"}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper(config, http_context, sessions, metrics, 'get_hostname')

@add_route('GET', '/dashboard/os_version')
def dashboard_os_version(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/os_version Operating system version
    @apiVersion 0.0.1
    @apiName GetDasboardOSVersion
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {String} os_version Operating system version.

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/os_version

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:49:19 GMT
        Content-type: application/json

        {"os_version": "Linux 3.16.0-34-generic x86_64"}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper(config, http_context, sessions, metrics, 'get_os_version')

@add_route('GET', '/dashboard/pg_version')
def dashboard_pg_version(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/pg_version PostgreSQL version
    @apiVersion 0.0.1
    @apiName GetDasboardPGVersion
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {String} pg_version PostgreSQL version.

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/pg_version

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:49:19 GMT
        Content-type: application/json

        {"pg_version": "9.4.1"}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper_pg(config, http_context, sessions, metrics, 'get_pg_version')

@add_route('GET', '/dashboard/n_cpu')
def dashboard_n_cpu(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/n_cpu Number of CPU
    @apiVersion 0.0.1
    @apiName GetDasboardNCpu
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Number} n_cpu Number of CPU.

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/n_cpu

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:49:19 GMT
        Content-type: application/json

        {"n_cpu": 4}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper(config, http_context, sessions, metrics, 'get_n_cpu')

@add_route('GET', '/dashboard/databases')
def dashboard_databases(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /dashboard/databases PostgreSQL instance size & number of DB
    @apiVersion 0.0.1
    @apiName GetDasboardDatabases
    @apiGroup Dashboard

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object} databases
    @apiSuccess {String} databases.total_size PostgreSQL instance total size (with unit).
    @apiSuccess {String} databases.time Time when DBs informations have been retreived (HH:MM).
    @apiSuccess {Number} databases.databases Number of databases.
    @apiSuccess {Number} databases.total_commit Number of commited xact.
    @apiSuccess {Number} databases.total_rollback Number of rollbacked xact.

    @apiExample {curl} Example usage:
        curl -H "X-Session: <session-id>" http://localhost:12345/dashboard/databases

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 11:52:05 GMT
        Content-type: application/json

        {"databases": {"total_size": "1242 MB", "time": "13:52", "databases": 4, "total_commit": 16728291, "total_rollback": 873}}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}
    """
    set_logger_name("dashboard")
    return api_function_wrapper_pg(config, http_context, sessions, metrics, 'get_databases')

@add_route('GET', '/dashboard/info')
def dashboard_info(http_context, queue_in = None, config = None, sessions = None, commands = None):
    set_logger_name("dashboard")
    return api_function_wrapper_pg(config, http_context, sessions, metrics, 'get_info')

def dashboard_worker_sigterm_handler(signum, frame):
    logging.info("Dashboard collector worker received SIGTERM")
    sys.exit(1)

@add_worker(b'dashboard_collector')
def dashboard_collector_worker(commands, command, config):
    try:
        signal.signal(signal.SIGTERM, dashboard_worker_sigterm_handler)
        start_time = time.time() * 1000
        set_logger_name("dashboard_collector")
        logger = get_logger(config)
        logger.debug("Starting with pid=%s" % (getpid()))
        logger.debug("commandid=%s" % (command.commandid))
        command.state = COMMAND_START
        command.time = time.time()
        command.pid = getpid()
        commands.update(command)

        conn = connector(
            host = config.postgresql['host'],
            port = config.postgresql['port'],
            user = config.postgresql['user'],
            password = config.postgresql['password'],
            database = config.postgresql['dbname']
        )
        conn.connect()
        db_metrics = metrics.get_metrics(conn, config)
        # We don't want to store notifications in the history.
        db_metrics.pop('notifications', None)

        conn.close()
        q = Queue('%s/dashboard.q'% (config.temboard['home']), max_length = (config.plugins['dashboard']['history_length']+1), overflow_mode = 'slide')
        q.push(Message(content = json.dumps(db_metrics)))
        logger.debug("Duration: %s." % (str(time.time() * 1000 - start_time)))
        logger.debug("Done.")
    except (error, Exception) as e:
        logger.traceback(get_tb())
        logger.error(str(e))
        logger.debug("Failed.")
        try:
            conn.close()
        except Exception:
            pass
        sys.exit(1)

def scheduler(queue_in, config, commands):
    logger = get_logger(config)
    worker = b'dashboard_collector'
    parameters = ''
    # Check command uniqueness.
    try:
        commands.check_uniqueness(worker, parameters)
    except SharedItem_exists:
        return

    cid =  hash_id(worker)
    command = Command(cid.encode('utf-8'), time.time(), 0, worker, parameters, 0, u'')
    try:
        commands.add(command)
        # Put the Command in the command queue
        queue_in.put(command)
        return
    except SharedItem_no_free_slot_left:
        return


def configuration(config):
    class Configuration(PluginConfiguration):
        def __init__(self, config, *args, **kwargs):
            PluginConfiguration.__init__(self, config.configfile, *args, **kwargs)

            self.plugin_configuration = {
                'scheduler_interval': 2,
                'history_length': 20
            }
            set_logger_name("dashboard")
            logger = get_logger(config)

            try:
                self.check_section(__name__)
            except ConfigurationError as e:
                return

            try:
                if not (self.getint(__name__, 'scheduler_interval') > 0 and \
                    self.getint(__name__, 'scheduler_interval') < 86400):
                    raise ValueError()
                self.plugin_configuration['scheduler_interval'] = \
                    self.getint(__name__, 'scheduler_interval')
            except ValueError as e:
                logger.error("%s - configuration error: 'scheduler_interval' must be"
                    "an integer between 0 and 86400 in '%s' section in %s."
                    % (__name__, self.configfile, __name__))
            except NoOptionError as e:
                pass

            try:
                if not (self.getint(__name__, 'history_length') > 0 and \
                    self.getint(__name__, 'history_length') < 300):
                    raise ValueError()
                self.plugin_configuration['history_length'] = \
                    self.getint(__name__, 'history_length')
            except ValueError as e:
                logger.error("%s - configuration error: 'histor_length' must be"
                    "an integer between 0 and 300 in '%s' section in %s."
                    % (__name__, self.configfile, __name__))
            except NoOptionError as e:
                pass

    c = Configuration(config)
    return c.plugin_configuration



from temboardagent.routing import add_route, add_worker
from temboardagent.api_wrapper import *
from temboardagent.logger import set_logger_name
import activity.functions as activity_functions

@add_route('GET', '/activity')
def get_activity(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /activity Get the list of backend.
    @apiVersion 0.0.1
    @apiName Activity
    @apiGroup Activity

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object[]} response.rows List of backends.
    @apiSuccess {Number}   response.rows.pid Process ID of this backend.
    @apiSuccess {String}   response.rows.database Name of the database this backend is connected to.
    @apiSuccess {String}   response.rows.user Name of the user logged into this backend.
    @apiSuccess {String}   response.rows.client IP address of the client connected to this backend.
    @apiSuccess {Number}   response.rows.cpu CPU usage (%) of this backend.
    @apiSuccess {Number}   response.rows.memory Memory usage (%) of this backend.
    @apiSuccess {String}   response.rows.read_s Read rate of this backend.
    @apiSuccess {String}   response.rows.write_s Write rate of this backend.
    @apiSuccess {String}   response.rows.iow Is this backend waiting for IO operations.
    @apiSuccess {String}   response.rows.w Is this backend waiting for lock acquisition.
    @apiSuccess {Number}   response.rows.duration Query duration (s).
    @apiSuccess {String}   response.rows.state State of this backend.
    @apiSuccess {String}   response.rows.query Query.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/activity

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:57:52 GMT
        Content-type: application/json
        {
            "rows":
            [
                {
                    "pid": 6285,
                    "database": "postgres",
                    "user": "postgres",
                    "client": null,
                    "cpu": 0.0,
                    "memory": 0.13,
                    "read_s": "0.00B",
                    "write_s": "0.00B",
                    "iow": "N",
                    "wait": "N",
                    "duration": "1.900",
                    "state": "idle",
                    "query": "SELECT 1;"
                },
                ...
            ]
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """
    set_logger_name('activity')
    return api_function_wrapper_pg(config, http_context, sessions, activity_functions, 'get_activity')

@add_route('GET', '/activity/waiting')
def get_activity_waiting(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /activity/waiting Get the list of backend waiting for lock acquisition.
    @apiVersion 0.0.1
    @apiName ActivityWaiting
    @apiGroup Activity

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object[]} response.rows List of backends.
    @apiSuccess {Number}   response.rows.pid Process ID of this backend.
    @apiSuccess {String}   response.rows.database Name of the database this backend is connected to.
    @apiSuccess {String}   response.rows.user Name of the user logged into this backend.
    @apiSuccess {Number}   response.rows.cpu CPU usage (%) of this backend.
    @apiSuccess {Number}   response.rows.memory Memory usage (%) of this backend.
    @apiSuccess {String}   response.rows.read_s Read rate of this backend.
    @apiSuccess {String}   response.rows.write_s Write rate of this backend.
    @apiSuccess {String}   response.rows.iow Is this backend waiting for IO operations.
    @apiSuccess {Number}   response.rows.relation OID of the relation targeted by the lock.
    @apiSuccess {Number}   response.rows.type Type of lockable object.
    @apiSuccess {Number}   response.rows.mode Name of the lock mode held or desired by this process.
    @apiSuccess {Number}   response.rows.duration Query duration (s).
    @apiSuccess {String}   response.rows.state State of this backend.
    @apiSuccess {String}   response.rows.query Query.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/activity/waiting

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:57:52 GMT
        Content-type: application/json
        {
            "rows":
            [
                {
                    "pid": 13532,
                    "database": "test",
                    "user": "postgres",
                    "cpu": 0.0,
                    "memory": 0.16,
                    "read_s": "0.00B",
                    "write_s": "0.00B",
                    "iow": "N",
                    "relation": " ",
                    "type": "transactionid",
                    "mode": "ShareLock",
                    "state": "active",
                    "duration": 4.35,
                    "query": "DELETE FROM t1 WHERE id = 1;"
                },
                ...
            ]
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """
    set_logger_name('activity')
    return api_function_wrapper_pg(config, http_context, sessions, activity_functions, 'get_activity_waiting')

@add_route('GET', '/activity/blocking')
def get_activity_blocking(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {get} /activity/blocking Get the list of backend blocking other backends due to lock acquisition.
    @apiVersion 0.0.1
    @apiName ActivityBlocking
    @apiGroup Activity

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object[]} response.rows List of backends.
    @apiSuccess {Number}   response.rows.pid Process ID of this backend.
    @apiSuccess {String}   response.rows.database Name of the database this backend is connected to.
    @apiSuccess {String}   response.rows.user Name of the user logged into this backend.
    @apiSuccess {Number}   response.rows.cpu CPU usage (%) of this backend.
    @apiSuccess {Number}   response.rows.memory Memory usage (%) of this backend.
    @apiSuccess {String}   response.rows.read_s Read rate of this backend.
    @apiSuccess {String}   response.rows.write_s Write rate of this backend.
    @apiSuccess {String}   response.rows.iow Is this backend waiting for IO operations.
    @apiSuccess {Number}   response.rows.relation OID of the relation targeted by the lock.
    @apiSuccess {Number}   response.rows.type Type of lockable object.
    @apiSuccess {Number}   response.rows.mode Name of the lock mode held or desired by this process.
    @apiSuccess {String}   response.rows.state State of this backend.
    @apiSuccess {Number}   response.rows.duration Query duration (s).
    @apiSuccess {String}   response.rows.query Query.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/activity/blocking

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:57:52 GMT
        Content-type: application/json
        {
            "rows":
            [
                {
                    "pid": 13309,
                    "database": "test",
                    "user": "postgres",
                    "cpu": 0.0,
                    "memory": 0.2,
                    "read_s": "0.00B",
                    "write_s": "0.00B",
                    "iow": "N",
                    "relation": " ",
                    "type": "transactionid",
                    "mode": "ExclusiveLock",
                    "state": "idle in transaction",
                    "duration": 4126.98,
                    "query": "UPDATE t1 SET id = 100000000 where id =1;"
                },
                ...
            ]
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """
    set_logger_name('activity')
    return api_function_wrapper_pg(config, http_context, sessions, activity_functions, 'get_activity_blocking')

@add_route('POST', '/activity/kill')
def post_activity_kill(http_context, queue_in = None, config = None, sessions = None, commands = None):
    """
    @api {post} /activity/kill Terminate N backends.
    @apiVersion 0.0.1
    @apiName ActivityKill
    @apiGroup Activity

    @apiHeader {String} X-Session Session ID.
    @apiParam  {String[]} pids List of process ID to terminate.

    @apiSuccess {Object[]} response.backends List of backend status.
    @apiSuccess {Number}   response.backends.pid Process ID of this backend.
    @apiSuccess {Boolean}  response.backends.killes Was the backend killed ?

    @apiExample {curl} Example usage:
        curl -k -X POST -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
            -H "Content-Type: application/json" --data '{"pids": [ 13309 ]}' \
            "https://localhost:2345/activity/kill"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:57:52 GMT
        Content-type: application/json
        {
            "backends":
            [
                {"pid": 13309, "killed": true},
                ...
            ]
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """
    set_logger_name('activity')
    return api_function_wrapper_pg(config, http_context, sessions, activity_functions, 'post_activity_kill')



import time

from temboardagent.routing import add_route
from temboardagent.errors import (
    HTTPError,
    NotificationError,
    SharedItem_exists,
    SharedItem_no_free_slot_left,
    SharedItem_not_found,
)
from temboardagent.sharedmemory import Session
from temboardagent.types import (
    T_COMMANDID,
    T_PASSWORD,
    T_SESSIONID,
    T_USERNAME,
)
from temboardagent.tools import validate_parameters
from temboardagent.usermgmt import auth_user, gen_sessionid
from temboardagent.logger import get_logger, set_logger_name, get_tb
from temboardagent.spc import connector, error
from temboardagent.workers import COMMAND_DONE, COMMAND_ERROR
from temboardagent.notification import NotificationMgmt, Notification
from temboardagent.inventory import SysInfo, PgInfo


def check_sessionid(http_header, sessions):
    """
    Check X-Session is valid and the session exists.
    """
    validate_parameters(http_header,
                        [('X-Session', T_SESSIONID, False)])
    try:
        session = sessions.get_by_sessionid(
                    http_header['X-Session'].encode('utf-8'))
        session.time = time.time()
        username = session.username
        sessions.update(session)
        return username
    except SharedItem_not_found:
        raise HTTPError(401, "Invalid session.")


@add_route('POST', '/login')
def login(http_context, queue_in=None, config=None, sessions=None,
          commands=None):
    """
    @api {get} /login User login
    @apiVersion 0.0.1
    @apiName UserLogin
    @apiGroup User

    @apiParam {String} username Username.
    @apiParam {String} password Password.

    @apiSuccess {String} sessions Session ID.

    @apiExample {curl} Example usage:
        curl -k -X POST -H "Content-Type: application/json" -d '{"username": "julien", "password": "password12!"}' \
            https://localhost:2345/login

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:19:48 GMT
        Content-type: application/json

        {"session": "fa452548403ac53f2158a65f5eb6db9723d2b07238dd83f5b6d9ca52ce817b63"}

    @apiError (500 error) error Internal error.
    @apiError (404 error) error Invalid username or password.
    @apiError (406 error) error Username or password malformed or missing.

    @apiErrorExample 404 error example
        HTTP/1.0 404 Not Found
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:20:33 GMT
        Content-type: application/json

        {"error": "Invalid username/password."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:21:01 GMT
        Content-type: application/json

        {"error": "Parameter 'password' is malformed."}
    """  # noqa
    post = http_context['post']
    set_logger_name("api")
    logger = get_logger(config)
    # Add an unconditional sleeping time to reduce brute-force risks
    time.sleep(1)

    logger.info("Authenticating user: %s" % (post['username']))
    try:
        validate_parameters(post,
                            [('username', T_USERNAME, False),
                             ('password', T_PASSWORD, False)])
        auth_user(config.temboard['users'],
                  post['username'],
                  post['password'])
    except HTTPError as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        logger.info("Authentication failed.")
        raise e
    try:
        session = sessions.get_by_username(post['username'])
        if not session:
            sessionid = gen_sessionid(post['username'])
            session = Session(sessionid.encode('utf-8'),
                              time.time(),
                              post['username'].encode('utf-8'))
            sessions.add(session)
        else:
            sessionid = session.sessionid
            session.time = time.time()
            sessions.update(session)
        try:
            NotificationMgmt.push(config, Notification(
                                        username=post['username'],
                                        message="Login"))
        except NotificationError as e:
            logger.traceback(get_tb())
            logger.error(e.message)

    except (SharedItem_exists, SharedItem_no_free_slot_left) as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        raise HTTPError(500, "Internal error.")
    return {'session': sessionid}


@add_route('GET', '/logout')
def logout(http_context, queue_in=None, config=None, sessions=None,
           commands=None):
    """
    @api {get} /logout User logout
    @apiVersion 0.0.1
    @apiName UserLogout
    @apiGroup User

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Bool} logout True if logout succeeds.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: fa452548403ac53f2158a65f5eb6db9723d2b07238dd83f5b6d9ca52ce817b63" https://localhost:2345/logout

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:33:19 GMT
        Content-type: application/json

        {"logout": true}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session ID.
    @apiError (406 error) error Session ID malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:36:33 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:37:23 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}
    """  # noqa
    headers = http_context['headers']
    set_logger_name("api")
    logger = get_logger(config)
    logger.info("Removing session: %s" % (headers['X-Session']))
    try:
        username = check_sessionid(headers, sessions)
    except HTTPError as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        logger.info("Invalid session.")
        raise e

    try:
        NotificationMgmt.push(config, Notification(
                                        username=username,
                                        message="Logout"))
    except NotificationError as e:
        logger.traceback(get_tb())
        logger.error(e.message)

    try:
        sessions.delete(headers['X-Session'].encode('utf-8'))
    except (SharedItem_exists, SharedItem_no_free_slot_left) as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        raise HTTPError(500, "Internal error.")
    return {'logout': True}


@add_route('GET', '/discover')
def get_discover(http_contexte, queue_in=None, config=None, sessions=None,
                 commands=None):
    """
    @api {get} /discover Get global informations about the env.
    @apiVersion 0.0.1
    @apiName Discover
    @apiGroup User

    @apiSuccess {String}   hostname Hostname.
    @apiSuccess {String}   pg_data PostgreSQL data directory.
    @apiSuccess {Number}   pg_port PostgreSQL listen port.
    @apiSuccess {String}   pg_version PostgreSQL version.
    @apiSuccess {String[]} plugins List or available plugins.
    @apiSuccess {Number}   memory_size Memory size (bytes).
    @apiSuccess {Number}   cpu Number of CPU.

    @apiExample {curl} Example usage:
        curl -k https://localhost:2345/discover

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:33:19 GMT
        Content-type: application/json
        {
            "hostname": "neptune",
            "pg_data": "/var/lib/postgresql/9.4/main",
            "pg_port": 5432,
            "plugins": ["monitoring", "dashboard", "pgconf",
                        "administration", "activity"],
            "memory_size": 8241508352,
            "pg_version": "PostgreSQL 9.4.5 on x86_64-unknown-linux-gnu, compiled by gcc (Ubuntu 4.9.2-10ubuntu13) 4.9.2, 64-bit",
            "cpu": 4
        }

    @apiError (500 error) error Internal error.
    """  # noqa
    set_logger_name("api")
    logger = get_logger(config)
    conn = connector(
        host=config.postgresql['host'],
        port=config.postgresql['port'],
        user=config.postgresql['user'],
        password=config.postgresql['password'],
        database=config.postgresql['dbname']
    )
    logger.info('Starting discovery.')
    try:
        conn.connect()
        sysinfo = SysInfo()
        pginfo = PgInfo(conn)
        ret = {
            'hostname': sysinfo.hostname(config.temboard['hostname']),
            'cpu': sysinfo.n_cpu(),
            'memory_size': sysinfo.memory_size(),
            'pg_port': pginfo.setting('port'),
            'pg_version': pginfo.version()['full'],
            'pg_data': pginfo.setting('data_directory'),
            'plugins': [plugin_name for plugin_name in
                        config.temboard['plugins']]
        }
        conn.close()
        logger.info('Discovery done.')
        return ret

    except (error, Exception, HTTPError) as e:
        logger.traceback(get_tb())
        logger.error(str(e))
        logger.info('Discovery failed.')
        try:
            conn.close()
        except Exception:
            pass
        if isinstance(e, HTTPError):
            raise e
        else:
            raise HTTPError(500, "Internal error.")


@add_route('GET', '/profile')
def profile(http_context, queue_in=None, config=None, sessions=None,
            commands=None):
    """
    @api {get} /profile Get current user name.
    @apiVersion 0.0.1
    @apiName Profile
    @apiGroup User

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {String} username Username.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: fa452548403ac53f2158a65f5eb6db9723d2b07238dd83f5b6d9ca52ce817b63" https://localhost:2345/profile

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:33:19 GMT
        Content-type: application/json
        {
            "username": "julien"
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session ID.
    @apiError (406 error) error Session ID malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:36:33 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:37:23 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}
    """  # noqa
    headers = http_context['headers']
    set_logger_name("api")
    logger = get_logger(config)
    logger.info("Get user profile.")
    try:
        check_sessionid(headers, sessions)
    except HTTPError as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        logger.info("Invalid session.")
        raise e
    try:
        session = sessions.get_by_sessionid(
                    headers['X-Session'].encode('utf-8'))
        logger.info("Done.")
        return {'username': session.username}
    except SharedItem_not_found as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        logger.info("Failed.")
        raise HTTPError(401, "Invalid session.")


@add_route('GET', '/command/'+T_COMMANDID)
def get_command(http_context, queue_in=None, config=None, sessions=None,
                commands=None):
    headers = http_context['headers']
    set_logger_name("api")
    logger = get_logger(config)
    logger.info("Get command status.")
    try:
        check_sessionid(headers, sessions)
    except HTTPError as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        logger.info("Invalid session.")
        raise e
    cid = http_context['urlvars'][0]
    try:
        command = commands.get_by_commandid(cid.encode('utf-8'))
        c_time = command.time
        c_state = command.state
        c_result = command.result
        if c_state == COMMAND_DONE or c_state == COMMAND_ERROR:
            commands.delete(cid.encode('utf-8'))
        logger.info("Done.")
        return {'cid': cid,
                'time': c_time,
                'state': c_state,
                'result': c_result}
    except SharedItem_not_found as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        logger.info("Failed.")
        raise HTTPError(401, "Invalid command.")


@add_route('GET', '/notifications')
def notifications(http_context, queue_in=None, config=None, sessions=None,
                  commands=None):
    """
    @api {get} /notifications Get all notifications.
    @apiVersion 0.0.1
    @apiName Notifications
    @apiGroup User

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object[]} notifications List of notifications.
    @apiSuccess {String}   notifications.date Notification datetime.
    @apiSuccess {String}   notifications.username Username.
    @apiSuccess {String}   notifications.message Message.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: fa452548403ac53f2158a65f5eb6db9723d2b07238dd83f5b6d9ca52ce817b63" https://localhost:2345/notifications

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:33:19 GMT
        Content-type: application/json

        [
            {"date": "2016-04-11T16:12:38", "username": "julien", "message": "Login"},
            {"date": "2016-04-11T16:02:03", "username": "julien", "message": "Login"},
            {"date": "2016-04-11T15:51:15", "username": "julien", "message": "HBA file version '2016-04-11T15:32:53' removed."},
            {"date": "2016-04-11T15:51:10", "username": "julien", "message": "HBA file version '2016-04-11T15:47:26' removed."},
            {"date": "2016-04-11T15:51:04", "username": "julien", "message": "HBA file version '2016-04-11T15:48:50' removed."},
            {"date": "2016-04-11T15:50:57", "username": "julien", "message": "PostgreSQL reload"},
            {"date": "2016-04-11T15:50:57", "username": "julien", "message": "HBA file updated"},
            {"date": "2016-04-11T15:48:50", "username": "julien", "message": "PostgreSQL reload"}
        ]

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session ID.
    @apiError (406 error) error Session ID malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:36:33 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 12:37:23 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}
    """  # noqa
    headers = http_context['headers']
    set_logger_name("api")
    logger = get_logger(config)
    logger.info("Get notifications.")
    try:
        check_sessionid(headers, sessions)
    except HTTPError as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        logger.info("Invalid session.")
        raise e

    try:
        notifications = NotificationMgmt.get_last_n(config, -1)
        logger.info("Done.")
        return list(notifications)
    except (NotificationError, Exception) as e:
        logger.traceback(get_tb())
        logger.error(e.message)
        logger.info("Failed.")
        raise HTTPError(500, "Internal error.")



from activity.process import *
from resource import getpagesize
from temboardagent.notification import NotificationMgmt, Notification
from temboardagent.tools import validate_parameters
from temboardagent.types import *
from temboardagent.errors import NotificationError

def get_activity(conn, config, _):
    mem_total = memory_total_size()
    page_size = getpagesize()
    if conn.get_pg_version() >= 90600:
        conn.execute("""
    SELECT
        pg_stat_activity.pid AS pid,
        pg_stat_activity.datname AS database,
        pg_stat_activity.client_addr AS client,
        round(EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start))::numeric, 2)::FLOAT AS duration,
        CASE WHEN pg_stat_activity.wait_event_type IS DISTINCT FROM 'Lock' THEN 'N' ELSE 'Y' END AS wait,
        pg_stat_activity.usename AS user,
        pg_stat_activity.state AS state,
        pg_stat_activity.query AS query
    FROM
        pg_stat_activity
    WHERE
        pid <> pg_backend_pid()
        -- AND state <> 'idle'
    ORDER BY
        EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) DESC
        """)
    else:
        conn.execute("""
    SELECT
        pg_stat_activity.pid AS pid,
        pg_stat_activity.datname AS database,
        pg_stat_activity.client_addr AS client,
        round(EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start))::numeric, 2)::FLOAT AS duration,
        CASE WHEN pg_stat_activity.waiting = 't' THEN 'Y' ELSE 'N' END AS wait,
        pg_stat_activity.usename AS user,
        pg_stat_activity.state AS state,
        pg_stat_activity.query AS query
    FROM
        pg_stat_activity
    WHERE
        pid <> pg_backend_pid()
        -- AND state <> 'idle'
    ORDER BY
        EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) DESC
        """)
    backend_list = []
    for row in conn.get_rows():
        try:
            backend_list.append({
                'pid': row['pid'],
                'database': row['database'],
                'client': row['client'],
                'duration': row['duration'],
                'wait': row['wait'],
                'user': row['user'],
                'state': row['state'],
                'query': row['query'],
                'process': Process(row['pid'], mem_total, page_size)})
        except Exception as e:
            pass

    time.sleep(0.1)
    final_backend_list = []
    for row in backend_list:
        try:
            (read_s, write_s) = row['process'].io_usage()
            if row['duration'] < 0:
                row['duration'] = 0
            final_backend_list.append({
                'pid': row['pid'],
                'database': row['database'],
                'client': row['client'],
                'duration': row['duration'],
                'wait': row['wait'],
                'user': row['user'],
                'state': row['state'],
                'query': row['query'],
                'iow': row['process'].iow,
                'read_s': read_s,
                'write_s': write_s,
                'cpu': row['process'].cpu_usage(),
                'memory': row['process'].mem_usage()
            })
        except Exception as e:
            pass
    return {'rows':final_backend_list}

def post_activity_kill(conn, config, http_context):
    validate_parameters(http_context['post'], [
        ('pids', T_PID, True)
    ])
    ret = {'backends': []}
    for pid in http_context['post']['pids']:
        conn.execute("SELECT pg_terminate_backend(%s) AS killed" % (pid))
        # Push a notification.
        try:
            NotificationMgmt.push(config, Notification(
                                username = http_context['username'],
                                message = "Backend %s terminated" % (pid)))
        except (NotificationError, Exception) as e:
            pass

        ret['backends'].append({'pid':pid, 'killed': list(conn.get_rows())[0]['killed']})
    return ret

def get_activity_waiting(conn, config, _):
    mem_total = memory_total_size()
    page_size = getpagesize()

    conn.execute("""
    SELECT
        pg_locks.pid AS pid,
        pg_stat_activity.datname AS database,
        pg_stat_activity.usename AS user,
        pg_locks.mode AS mode,
        pg_locks.locktype AS type,
        COALESCE(pg_locks.relation::regclass::text, ' ') AS relation,
        round(EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start))::numeric,2)::FLOAT AS duration,
        pg_stat_activity.state AS state,
        pg_stat_activity.query AS query
    FROM
        pg_catalog.pg_locks
        JOIN pg_catalog.pg_stat_activity ON(pg_catalog.pg_locks.pid = pg_catalog.pg_stat_activity.pid)
    WHERE
        NOT pg_catalog.pg_locks.granted
        AND pg_catalog.pg_stat_activity.pid <> pg_backend_pid()
    ORDER BY
        EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) DESC
    """)
    backend_list = []
    for row in conn.get_rows():
        try:
            backend_list.append({
                'pid': row['pid'],
                'database': row['database'],
                'user': row['user'],
                'mode': row['mode'],
                'type': row['type'],
                'relation': row['relation'],
                'duration': row['duration'],
                'state': row['state'],
                'query': row['query'],
                'process': Process(row['pid'], mem_total, page_size)})
        except Exception as e:
            pass

    time.sleep(0.1)
    final_backend_list = []
    for row in backend_list:
        try:
            (read_s, write_s) = row['process'].io_usage()
            if row['duration'] < 0:
                row['duration'] = 0
            final_backend_list.append({
                'pid': row['pid'],
                'database': row['database'],
                'user': row['user'],
                'mode': row['mode'],
                'type': row['type'],
                'relation': row['relation'],
                'duration': row['duration'],
                'state': row['state'],
                'query': row['query'],
                'iow': row['process'].iow,
                'read_s': read_s,
                'write_s': write_s,
                'cpu': row['process'].cpu_usage(),
                'memory': row['process'].mem_usage()
            })
        except Exception as e:
            pass
    return {'rows':final_backend_list}

def get_activity_blocking(conn, config, _):
    mem_total = memory_total_size()
    page_size = getpagesize()

    conn.execute("""
    SELECT
        pid,
        datname AS database,
        usename AS user,
        COALESCE(relation::text, ' ') AS relation,
        mode,
        locktype AS type,
        duration,
        state,
        query
    FROM
        (
        SELECT
            blocking.pid,
            pg_stat_activity.query,
            blocking.mode,
            pg_stat_activity.datname,
            pg_stat_activity.usename,
            blocking.locktype,
            round(EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start))::numeric,2)::FLOAT AS duration,
            blocking.relation::regclass AS relation,
            pg_stat_activity.state AS state
        FROM
            pg_locks AS blocking
            JOIN (
                SELECT
                    transactionid
                FROM
                    pg_locks
                WHERE
                    NOT granted) AS blocked ON (blocking.transactionid = blocked.transactionid)
            JOIN pg_stat_activity ON (blocking.pid = pg_stat_activity.pid)
        WHERE
            blocking.granted
        UNION ALL
        SELECT
            blocking.pid,
            pg_stat_activity.query,
            blocking.mode,
            pg_stat_activity.datname,
            pg_stat_activity.usename,
            blocking.locktype,
            round(EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start))::numeric,2)::FLOAT AS duration,
            blocking.relation::regclass AS relation,
            pg_stat_activity.state AS state
        FROM
            pg_locks AS blocking
            JOIN (
                SELECT
                    database,
                    relation,
                    mode
                FROM
                    pg_locks
                WHERE
                    NOT granted
                    AND relation IS NOT NULL) AS blocked ON (blocking.database = blocked.database AND blocking.relation = blocked.relation)
            JOIN pg_stat_activity ON (blocking.pid = pg_stat_activity.pid)
        WHERE
            blocking.granted
        ) AS sq
    GROUP BY
        pid,
        query,
        mode,
        locktype,
        duration,
        datname,
        usename,
        relation,
        state
    ORDER BY
        duration DESC
    """)
    backend_list = []
    for row in conn.get_rows():
        try:
            backend_list.append({
                'pid': row['pid'],
                'database': row['database'],
                'user': row['user'],
                'mode': row['mode'],
                'type': row['type'],
                'relation': row['relation'],
                'duration': row['duration'],
                'state': row['state'],
                'query': row['query'],
                'process': Process(row['pid'], mem_total, page_size)})
        except Exception as e:
            pass

    time.sleep(0.1)
    final_backend_list = []
    for row in backend_list:
        try:
            (read_s, write_s) = row['process'].io_usage()
            if row['duration'] < 0:
                row['duration'] = 0
            final_backend_list.append({
                'pid': row['pid'],
                'database': row['database'],
                'user': row['user'],
                'mode': row['mode'],
                'type': row['type'],
                'relation': row['relation'],
                'duration': row['duration'],
                'state': row['state'],
                'query': row['query'],
                'iow': row['process'].iow,
                'read_s': read_s,
                'write_s': write_s,
                'cpu': row['process'].cpu_usage(),
                'memory': row['process'].mem_usage()
            })
        except Exception as e:
            pass
    return {'rows':final_backend_list}



from temboardagent.routing import add_route
from temboardagent.api_wrapper import (
    api_function_wrapper_pg,
)
from temboardagent.logger import set_logger_name
import pgconf.functions as pgconf_functions
from pgconf.types import (
    T_PGSETTINGS_CATEGORY,
)


@add_route('GET', '/pgconf/configuration')
@add_route('GET', '/settings/configuration')
def get_pg_configuration(http_context, queue_in=None, config=None,
                         sessions=None, commands=None):
    """
    @api {get} /pgconf/configuration Fetch all PostgreSQL settings.
    @apiVersion 0.0.1
    @apiName GetPgconfConfiguration
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object[]} response List of settings category.
    @apiSuccess {String}   response.category Category name.
    @apiSuccess {Object[]} response.rows List of settings.
    @apiSuccess {String}   response.rows.context Context required to set the parameter's value.
    @apiSuccess {String[]} response.rows.enum_vals Allowed values of an enum parameter (null for non-enum values).
    @apiSuccess {Number}   response.rows.min_val Minimum allowed value of the parameter (null for non-numeric values).
    @apiSuccess {Number}   response.rows.max_val Maximum allowed value of the parameter (null for non-numeric values).
    @apiSuccess {String}   response.rows.vartype Parameter type (bool, enum, integer, real, or string).
    @apiSuccess {String}   response.rows.auto_val Parameter value set in auto configuration file.
    @apiSuccess {String}   response.rows.auto_val_raw Parameter value set in auto configuration file (raw).
    @apiSuccess {String}   response.rows.file_val Parameter value set in main postgresql.conf file.
    @apiSuccess {String}   response.rows.file_val_raw Parameter value set in main postgresql.conf file (raw).
    @apiSuccess {String}   response.rows.boot_val Parameter value assumed at server startup if the parameter is not otherwise set.
    @apiSuccess {String}   response.rows.unit Implicit unit of the parameter.
    @apiSuccess {String}   response.rows.desc Parameter description.
    @apiSuccess {String}   response.rows.name Parameter name.
    @apiSuccess {String}   response.rows.setting Parameter value.
    @apiSuccess {String}   response.rows.setting_raw Parameter value (raw).

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/configuration

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:57:52 GMT
        Content-type: application/json
        [
            {
                "category": "Autovacuum",
                "rows":
                [
                    {
                        "context": "sighup",
                        "enumvals": null,
                        "max_val": null,
                        "vartype": "bool",
                        "auto_val": "off",
                        "auto_val_raw": "off",
                        "boot_val": "on",
                        "unit": null,
                        "desc": "Starts the autovacuum subprocess.",
                        "name": "autovacuum",
                        "min_val": null,
                        "setting": "off",
                        "setting_raw": "off",
                        "file_val": null,
                        "file_val_raw": null
                    },
                    ...
                ]
            },
            ...
        ]

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'get_settings')


@add_route('GET', '/pgconf/configuration/categories')
@add_route('GET', '/settings/configuration/categories')
def get_pg_configuration_categories(http_context, queue_in=None, config=None,
                                    sessions=None, commands=None):
    """
    @api {get} /pgconf/configuration/categories Fetch settings categories names.
    @apiVersion 0.0.1
    @apiName GetPgconfConfigurationCategories
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {String[]} categories List of settings category name.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/configuration/categories

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:57:52 GMT
        Content-type: application/json
        {
            "categories":
            [
                "Autovacuum",
                "Client Connection Defaults / Locale and Formatting",
                "Client Connection Defaults / Other Defaults",
                ...
            ]
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'get_settings_categories')


@add_route('POST', '/pgconf/configuration')
@add_route('POST', '/settings/configuration')
def post_pg_configuration(http_context, queue_in=None, config=None,
                          sessions=None, commands=None):
    """
    @api {post} /pgconf/configuration Update setting/s value.
    @apiVersion 0.0.1
    @apiName PostPgconfConfiguration
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiParam {Object[]} settings List of settings.
    @apiParam {String}   settings.name Setting name.
    @apiParam {String}   settings.setting Setting value.

    @apiSuccess {Object[]} settings List of settings updated.
    @apiSuccess {String}   settings.setting Setting value.
    @apiSuccess {Boolean}  settings.restart Does PostgreSQL need to be restarted.
    @apiSuccess {String}   settings.name Setting name.
    @apiSuccess {String}   settings.previous_setting Previous setting value.

    @apiExample {curl} Example usage:
        curl -k -X POST -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    -H "Content-Type: application/json" \
                    --data '{"settings": [{"name": "autovacuum", "setting": "on"}]}' \
                    https://localhost:2345/pgconf/configuration

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:57:52 GMT
        Content-type: application/json
        {
            "settings":
            [
                {
                    "setting": "on",
                    "restart": false,
                    "name": "autovacuum",
                    "previous_setting": "off"
                }
            ]
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.
    @apiError (406 error) error Setting item malformed.
    @apiError (400 error) error Invalid json format.


    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}


    @apiErrorExample 400 error example
        HTTP/1.0 400 Bad request
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid json format: Expecting ',' delimiter: line 1 column 51 (char 50)."}


    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'post_settings')


@add_route('GET', '/pgconf/configuration/category/'+T_PGSETTINGS_CATEGORY)
@add_route('GET', '/settings/configuration/category/'+T_PGSETTINGS_CATEGORY)
def get_pg_configuration_category(http_context, queue_in=None, config=None,
                                  sessions=None, commands=None):
    """
    @api {get} /pgconf/configuration/category/:categoryname Fetch settings for one category, based on categoryname.
    @apiVersion 0.0.1
    @apiName GetPgconfConfigurationCategory
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiParam  {String} categoryname Category Name.

    @apiSuccess {Object[]} response List of settings category.
    @apiSuccess {String}   response.category Category name.
    @apiSuccess {Object[]} response.rows List of settings.
    @apiSuccess {String}   response.rows.context Context required to set the parameter's value.
    @apiSuccess {String[]} response.rows.enum_vals Allowed values of an enum parameter (null for non-enum values).
    @apiSuccess {Number}   response.rows.min_val Minimum allowed value of the parameter (null for non-numeric values).
    @apiSuccess {Number}   response.rows.max_val Maximum allowed value of the parameter (null for non-numeric values).
    @apiSuccess {String}   response.rows.vartype Parameter type (bool, enum, integer, real, or string).
    @apiSuccess {String}   response.rows.auto_val Parameter value set in auto configuration file.
    @apiSuccess {String}   response.rows.auto_val_raw Parameter value set in auto configuration file (raw).
    @apiSuccess {String}   response.rows.file_val Parameter value set in main postgresql.conf file.
    @apiSuccess {String}   response.rows.file_val_raw Parameter value set in main postgresql.conf file (raw).
    @apiSuccess {String}   response.rows.boot_val Parameter value assumed at server startup if the parameter is not otherwise set.
    @apiSuccess {String}   response.rows.unit Implicit unit of the parameter.
    @apiSuccess {String}   response.rows.desc Parameter description.
    @apiSuccess {String}   response.rows.name Parameter name.
    @apiSuccess {String}   response.rows.setting Parameter value.
    @apiSuccess {String}   response.rows.setting_raw Parameter value (raw).


    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/configuration/category/Autovacuum"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Wed, 10 Feb 2016 09:56:30 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        [
            {
                "category": "Autovacuum",
                "rows":
                [
                    {
                        "context": "sighup",
                        "enumvals": null,
                        "max_val": null,
                        "vartype": "bool",
                        "auto_val": "on",
                        "auto_val_raw": "on",
                        "boot_val": "on",
                        "unit": null,
                        "desc": "Starts the autovacuum subprocess. ",
                        "name": "autovacuum",
                        "min_val": null,
                        "setting": "on",
                        "setting_raw": "on",
                        "file_val": null,
                        "file_val_raw": null
                    },
                    ...
                ]
            }
        ]

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'get_settings')


@add_route('GET', '/pgconf/configuration/status')
@add_route('GET', '/settings/configuration/status')
def get_pg_configuration_status(http_context, queue_in=None, config=None,
                                sessions=None, commands=None):
    """
    @api {get} /pgconf/configuration/status Shows settings waiting for PostgreSQL reload and/or restart
    @apiVersion 0.0.1
    @apiName GetPgconfConfigurationStatus
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object}    response
    @apiSuccess {Boolean}   response.restart_pending Does PostgreSQL need to be restarted ?
    @apiSuccess {Object[]}  response.restart_changes List of settings.
    @apiSuccess {String}    response.restart_changes.context Context required to set the parameter's value.
    @apiSuccess {String[]}  response.restart_changes.enum_vals Allowed values of an enum parameter (null for non-enum values).
    @apiSuccess {Number}    response.restart_changes.min_val Minimum allowed value of the parameter (null for non-numeric values).
    @apiSuccess {Number}    response.restart_changes.max_val Maximum allowed value of the parameter (null for non-numeric values).
    @apiSuccess {String}    response.restart_changes.vartype Parameter type (bool, enum, integer, real, or string).
    @apiSuccess {String}    response.restart_changes.auto_val Parameter value set in auto configuration file.
    @apiSuccess {String}    response.restart_changes.auto_val_raw Parameter value set in auto configuration file (raw).
    @apiSuccess {String}    response.restart_changes.file_val Parameter value set in main postgresql.conf file.
    @apiSuccess {String}    response.restart_changes.file_val_raw Parameter value set in main postgresql.conf file (raw).
    @apiSuccess {String}    response.restart_changes.boot_val Parameter value assumed at server startup if the parameter is not otherwise set.
    @apiSuccess {String}    response.restart_changes.unit Implicit unit of the parameter.
    @apiSuccess {String}    response.restart_changes.desc Parameter description.
    @apiSuccess {String}    response.restart_changes.name Parameter name.
    @apiSuccess {String}    response.restart_changes.setting Parameter value.
    @apiSuccess {String}    response.restart_changes.setting_raw Parameter value (raw).
    @apiSuccess {String}    response.restart_changes.pending_val Pending value.
    @apiSuccess {Boolean}   response.reload_pending Does PostgreSQL need to be reloaded ?
    @apiSuccess {Object[]}  response.reload_changes List of settings.
    @apiSuccess {String}    response.reload_changes.context Context required to set the parameter's value.
    @apiSuccess {String[]}  response.reload_changes.enum_vals Allowed values of an enum parameter (null for non-enum values).
    @apiSuccess {Number}    response.reload_changes.min_val Minimum allowed value of the parameter (null for non-numeric values).
    @apiSuccess {Number}    response.reload_changes.max_val Maximum allowed value of the parameter (null for non-numeric values).
    @apiSuccess {String}    response.reload_changes.vartype Parameter type (bool, enum, integer, real, or string).
    @apiSuccess {String}    response.reload_changes.auto_val Parameter value set in auto configuration file.
    @apiSuccess {String}    response.reload_changes.auto_val_raw Parameter value set in auto configuration file (raw).
    @apiSuccess {String}    response.reload_changes.file_val Parameter value set in main postgresql.conf file.
    @apiSuccess {String}    response.reload_changes.file_val_raw Parameter value set in main postgresql.conf file (raw).
    @apiSuccess {String}    response.reload_changes.boot_val Parameter value assumed at server startup if the parameter is not otherwise set.
    @apiSuccess {String}    response.reload_changes.unit Implicit unit of the parameter.
    @apiSuccess {String}    response.reload_changes.desc Parameter description.
    @apiSuccess {String}    response.reload_changes.name Parameter name.
    @apiSuccess {String}    response.reload_changes.setting Parameter value.
    @apiSuccess {String}    response.reload_changes.setting_raw Parameter value (raw).
    @apiSuccess {String}    response.reload_changes.pending_val Pending value.



    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/configuration/status"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Wed, 10 Feb 2016 09:56:30 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        {
            "restart_changes":
            [
                {
                    "context": "postmaster",
                    "setting_raw": "128MB",
                    "enumvals": null,
                    "max_val": 1073741823,
                    "vartype": "integer",
                    "auto_val": 32768,
                    "file_val_raw": "128MB",
                    "boot_val": 1024,
                    "unit": "8kB",
                    "desc": "Sets the number of shared memory buffers used by the server. ",
                    "name": "shared_buffers",
                    "auto_val_raw": "256MB",
                    "min_val": 16,
                    "setting": 16384,
                    "file_val": 16384,
                    "pending_val": "256MB"
                }
            ],
            "restart_pending": true,
            "reload_pending": false,
            "reload_changes":
            [
            ]
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'get_settings_status')


@add_route('GET', '/pgconf/hba')
@add_route('GET', '/settings/hba')
def get_pg_hba(http_context, queue_in=None, config=None, sessions=None,
               commands=None):
    """
    @api {get} /pgconf/hba?version=:version Get pg_hba.conf records
    @apiVersion 0.0.1
    @apiName GetPgconfHBA
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiParam  {String} version Version number, if omitted, current file is considered.

    @apiSuccess {Object[]}  response.entries HBA records list.
    @apiSuccess {String}    response.entries.connection The record matches connection attempts using Unix-domain sockets (local) or using TCP/IP (host) or using TCP/IP, but only when the connection is made with SSL encryption (hostssl) or using TCP/IP that do not use SSL.
    @apiSuccess {String}    response.entries.database Specifies which database name(s) this record matches.
    @apiSuccess {String}    response.entries.user Specifies which user name(s) this record matches.
    @apiSuccess {String}    response.entries.address Specifies the client machine address(es) that this record matches. Hostname or IP address range.
    @apiSuccess {String}    response.entries.auth_method Specifies the authentication method to use when a connection matches this record.
    @apiSuccess {String}    response.entries.auth_options Specifies options for the authentication method.
    @apiSuccess {String}    response.version pg_hba.conf file version, null if current.
    @apiSuccess {String}    response.filepath pg_hba.conf file path.


    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/hba"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Wed, 10 Feb 2016 09:56:30 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json*
        {
            "entries":
            [
                {
                    "database": "all",
                    "connection": "local",
                    "auth_options": "",
                    "user": "postgres",
                    "address": "",
                    "auth_method": "ident"
                },
                ...
            ],
            "version": null,
            "filepath": "/etc/postgresql/9.4/main/pg_hba.conf"
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.
    @apiError (404 error) error File version not found.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    @apiErrorExample 404 error example
        HTTP/1.0 404 Not Found
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Thu, 11 Feb 2016 09:04:02 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json

        {"error": "Version 2016-01-29T08:46:09 of file /etc/postgresql/9.4/main/pg_hba.conf does not exist."}

    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'get_hba')


@add_route('GET', '/pgconf/hba/raw')
@add_route('GET', '/settings/hba/raw')
def get_pg_hba_raw(http_context, queue_in=None, config=None, sessions=None,
                   commands=None):
    """
    @api {get} /pgconf/hba/raw?version=:version Get pg_hba.conf raw content
    @apiVersion 0.0.1
    @apiName GetPgconfHBARaw
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiParam  {String} version Version number, if omitted, current file is considered.

    @apiSuccess {Object}  response
    @apiSuccess {Object}  response.content pg_hba.conf file raw content.
    @apiSuccess {String}  response.version pg_hba.conf file version, null if current.
    @apiSuccess {String}  response.filepath pg_hba.conf file path.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/hba/raw?version=2016-01-29T08:46:09"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Thu, 11 Feb 2016 13:40:51 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        {
            "content": "# PostgreSQL Client Authentication Configuration File\r\n# ===================================================\r\n# Database administrative login by Unix domain socket\r\nlocal   all             postgres                           ident #tutu\r\nlocal   all             all                                md5 #toto\r\n# TYPE  DATABASE        USER            ADDRESS                 METHOD\r\n# \"local\" is for Unix domain socket connections only\r\nlocal   all             all                                     md5\r\n# IPv4 local connections:\r\nhost    all             all             127.0.0.1/32            md5\r\n# IPv6 local connections:\r\nhost    all             all             ::1/128                 md5\r\n# Allow replication connections from localhost, by a user with the\r\n# replication privilege.\r\nlocal   replication     postgres                                peer\r\nhost    replication     postgres        127.0.0.1/32            md5\r\nhost    replication     postgres        ::1/128                 md5\r\nhost\treplication\t\treplicator\t\t192.168.1.40/32\t\t\ttrust #test\r\n# test ok\r\nhost    all     all     192.168.1.0/24  trust\r\nhost    all     all     192.168.1.0/24  trust\r\n",
            "version": "2016-01-29T08:46:07",
            "filepath": "/etc/postgresql/9.4/main/pg_hba.conf"
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.
    @apiError (404 error) error File version not found.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    @apiErrorExample 404 error example
        HTTP/1.0 404 Not Found
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Thu, 11 Feb 2016 09:04:02 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json

        {"error": "Version 2016-01-29T08:46:09 of file /etc/postgresql/9.4/main/pg_hba.conf does not exist."}

    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'get_hba_raw')


@add_route('POST', '/pgconf/hba')
@add_route('POST', '/settings/hba')
def post_pg_hba(http_context, queue_in=None, config=None, sessions=None,
                commands=None):
    """
    @api {post} /pgconf/hba Replace pg_hba.conf file content.
    @apiVersion 0.0.1
    @apiName PostPgconfHBA
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiParam  {Object[]} entries List of records.
    @apiParam  {String}   entries.connection The record matches connection attempts using Unix-domain sockets (local) or using TCP/IP (host) or using TCP/IP, but only when the connection is made with SSL encryption (hostssl) or using TCP/IP that do not use SSL.
    @apiParam  {String}   entries.database Specifies which database name(s) this record matches.
    @apiParam  {String}   entries.user Specifies which user name(s) this record matches.
    @apiParam  {String}   entries.address Specifies the client machine address(es) that this record matches. Hostname or IP address range.
    @apiParam  {String}   entries.auth_method Specifies the authentication method to use when a connection matches this record.
    @apiParam  {String}   entries.auth_options Specifies options for the authentication method.
    @apiParam  {Boolean}  new_version Create a new version of current pg_hba.conf before writing new content.

    @apiSuccess {Object}  response
    @apiSuccess {String}  response.last_version pg_hba.conf last file version.
    @apiSuccess {String}  response.filepath pg_hba.conf file path.

    @apiExample {curl} Example usage:
        curl -k -X POST -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                        -H "Content-Type: application/json" \
                        --data '{"entries": [{"connection": "local", "user": "all", "database": "all", "auth_method": "peer"}, ... ], "new_version": true}' \
                    https://localhost:2345/pgconf/hba"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Thu, 11 Feb 2016 14:26:19 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        {
            "last_version": "2016-02-11T15:26:19",
            "filepath": "/etc/postgresql/9.4/main/pg_hba.conf"
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}


    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'post_hba')


@add_route('POST', '/pgconf/hba/raw')
@add_route('POST', '/settings/hba/raw')
def post_pg_hba_raw(http_context, queue_in=None, config=None, sessions=None,
                    commands=None):
    """
    @api {post} /pgconf/hba/raw Replace pg_hba.conf file content (raw mode).
    @apiVersion 0.0.1
    @apiName PostPgconfHBARaw
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiParam  {String}   content pg_hba.conf content (raw).
    @apiParam  {Boolean}  new_version Create a new version of current pg_hba.conf before writing new content.

    @apiSuccess {Object}  response
    @apiSuccess {String}  response.last_version pg_hba.conf last file version.
    @apiSuccess {String}  response.filepath pg_hba.conf file path.

    @apiExample {curl} Example usage:
        curl -k -X POST -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                        -H "Content-Type: application/json" \
                        --data '{"content": "local all all md5\r\n ...", "new_version": true}' \
                    https://localhost:2345/pgconf/hba/raw"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Thu, 11 Feb 2016 14:26:19 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        {
            "last_version": "2016-02-11T15:26:19",
            "filepath": "/etc/postgresql/9.4/main/pg_hba.conf"
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}


    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'post_hba_raw')


@add_route('DELETE', '/pgconf/hba')
@add_route('DELETE', '/settings/hba')
def delete_pg_hba(http_context, queue_in=None, config=None, sessions=None,
                  commands=None):
    """
    @api {delete} /pgconf/hba?version=:version Remove a previous pg_hba.conf version.
    @apiVersion 0.0.1
    @apiName DeletePgconfHBA
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiParam  {String} version Version number.

    @apiSuccess {Object}  response
    @apiSuccess {String}  response.version pg_hba.conf version.
    @apiSuccess {Boolean} response.deleted Is deleted ?

    @apiExample {curl} Example usage:
        curl -k -X DELETE -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/hba?version=2016-01-29T08:44:26"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Fri, 12 Feb 2016 09:54:17 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        {
            "deleted": true,
            "version": "2016-01-29T08:44:26"
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.
    @apiError (404 error) error File version not found.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    @apiErrorExample 404 error example
        HTTP/1.0 404 Not Found
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Fri, 12 Feb 2016 10:00:55 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json

        {"error": "Version 2016-01-29T08:44:26 of file /etc/postgresql/9.4/main/pg_hba.conf does not exist."}
    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'delete_hba_version')


@add_route('GET', '/pgconf/hba/versions')
@add_route('GET', '/settings/hba/versions')
def get_pg_hba_versions(http_context, queue_in=None, config=None,
                        sessions=None, commands=None):
    """
    @api {get} /pgconf/hba/versions Get the list of pg_hba.conf versions.
    @apiVersion 0.0.1
    @apiName GetPgconfHBAVersions
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object}   response
    @apiSuccess {String[]} response.versions List of versions, desc. sorting.
    @apiSuccess {String}   response.filepath pg_hba.conf file path.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/hba/versions"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Fri, 12 Feb 2016 10:38:10 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        {
            "versions":
            [
                "2016-02-11T18:01:35",
                "2016-02-11T16:43:51",
                "2016-02-11T16:43:36",
                ...
            ],
            "filepath": "/etc/postgresql/9.4/main/pg_hba.conf"}

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'get_hba_versions')


@add_route('GET', '/pgconf/pg_ident')
@add_route('GET', '/settings/pg_ident')
def get_pg_ident(http_context, queue_in=None, config=None, sessions=None,
                 commands=None):
    """
    @api {get} /pgconf/pg_ident Get pg_ident.conf raw content
    @apiVersion 0.0.1
    @apiName GetPgconfPGIdentRaw
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object}  response
    @apiSuccess {Object}  response.content pg_ident.conf file raw content.
    @apiSuccess {String}  response.filepath pg_ident.conf file path.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/pg_ident"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Fri, 12 Feb 2016 10:48:57 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        {
            "content": "# PostgreSQL User Name Maps\r\n# =========================\r\n ... ",
            "filepath": "/etc/postgresql/9.4/main/pg_ident.conf"
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'get_pg_ident')


@add_route('POST', '/pgconf/pg_ident')
@add_route('POST', '/settings/pg_ident')
def post_pg_ident(http_context, queue_in=None, config=None, sessions=None,
                  commands=None):
    """
    @api {post} /pgconf/pg_ident Replace pg_ident.conf file content (raw mode).
    @apiVersion 0.0.1
    @apiName PostPgconfPGIdentRaw
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiParam  {String}   content pg_ident.conf content (raw).

    @apiSuccess {Object}  response
    @apiSuccess {Boolean} response.update Has pg_ident.conf been updated ?

    @apiExample {curl} Example usage:
        curl -k -X POST -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                        -H "Content-Type: application/json" \
                        --data '{"content": "# PostgreSQL User Name Maps\r\n ..."}' \
                    https://localhost:2345/pgconf/pg_ident"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Thu, 11 Feb 2016 14:26:19 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        {
            "update": true
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}
    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'post_pg_ident')


@add_route('GET', '/pgconf/hba/options')
@add_route('GET', '/settings/hba/options')
def get_hba_options(http_context, queue_in=None, config=None, sessions=None,
                    commands=None):
    """
    @api {get} /pgconf/hba/options Get HBA potential values for each column.
    @apiVersion 0.0.1
    @apiName GetPgconfHBAOptions
    @apiGroup Pgconf

    @apiHeader {String} X-Session Session ID.

    @apiSuccess {Object}    response
    @apiSuccess {String[]}  response.connection Connection field potential values.
    @apiSuccess {String[]}  response.database Database field potential values.
    @apiSuccess {String[]}  response.user User field potential values.
    @apiSuccess {String[]}  response.auth_method Authentication methods.

    @apiExample {curl} Example usage:
        curl -k -H "X-Session: 3b28ed94743e3ada57b217bbf9f36c6d1eb45e669a1ab693e8ca7ac3bd070b9e" \
                    https://localhost:2345/pgconf/hba/options"

    @apiSuccessExample Success-Reponse:
        HTTP/1.0 200 OK
        Server: temboard-agent/0.0.1 Python/2.7.9
        Date: Fri, 12 Feb 2016 10:48:57 GMT
        Access-Control-Allow-Origin: *
        Content-type: application/json
        {
            "connection": [ "local", "host", "hostssl", "hostnossl" ],
            "database": [ "all", "sameuser", "samerole", "db1" ],
            "user": [ "all", "user1", "+group1" ],
            "auth_method": [ "trust", "reject", ... ]
        }

    @apiError (500 error) error Internal error.
    @apiError (401 error) error Invalid session.
    @apiError (406 error) error Parameter 'X-Session' is malformed.

    @apiErrorExample 401 error example
        HTTP/1.0 401 Unauthorized
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Invalid session."}

    @apiErrorExample 406 error example
        HTTP/1.0 406 Not Acceptable
        Server: temboard-agent/0.0.1 Python/2.7.8
        Date: Wed, 22 Apr 2015 09:58:00 GMT
        Content-type: application/json

        {"error": "Parameter 'X-Session' is malformed."}

    """  # noqa
    set_logger_name("pgconf")
    return api_function_wrapper_pg(config, http_context, sessions,
                                   pgconf_functions, 'get_hba_options')
